"""
*Authors:* [Yehya Abouelnaga](https://abouelnaga.io), Rishabh Raaj


This library has been developed as a part of a practical course in the [Technical University of Munich](https://www.in.tum.de/) <img src="https://upload.wikimedia.org/wikipedia/en/0/04/Technical_University_of_Munich_emblem.svg" style="max-height: 1em;" alt="Technical University of Munich" /> under the supervision of [Hagerer Gerhard](https://www.social.in.tum.de/en/team/gerhardhagerer/) and [Dr. Georg Groh](https://www.social.in.tum.de/en/team/georggroh/) (Research Group Social Copmuting).
During this project, we've been working on a "Multi-Instance Learning for Sentiment Analysis" problem.
Most of the training was happening on Google Colaboratory; Hence, we needed a portable version of our code.
That led to the creation of this library.

## Google Colaboratory Workflow
<div style="text-align: center;">
    <img src="https://i.ibb.co/wJQd1Jv/Untitled-Diagram.png" alt="Google Colab Workflow" border="0">
</div>

### Install the library on your Colab
```
pip install tum --extra-index-url http://pypi.abouelnaga.io --trusted-host pypi.abouelnaga.io
```

### Mount your Google Drive to Colab
```
! tum mount-google-drive
```

Note that we are using the `!` prefix; That helps us execute the previous command as a CLI command (inside Google Colab or Jupyter Notebook).

**Tip!** Define a local and an environment variable for your Google Drive storage location for your models and data.
That could be done using the following snippet:
```
import os
GS = "/content/gdrive/My Drive/Colab Notebooks"
os.environ["GS"] = GS
```

### Load Pre-Computed Data
In this section, I assume you are already familiar with the Command Line Interface (`tum.cli`) defined at this library.
I also assume that you pre-computed the following:

- Your labels (using `tum.cli.precompute_labels`)
- Your sentences (using `tum.cli.precompute_sentences`)
- Your bert embeddings (using `tum.cli.precompute_bert`)

Assuming that you generated your BERT embeddings and labels on the Amazon Fine Foods section, and that you stored your embeddings in (your mounted) Google Drive.

```
from tum.nlp import training
data = training.load_precomputed_data(
    total_chunks=400,
    X_path=lambda chunk_id: f"{GS}/finefoods.sentences.bert/finefoods.sentences.bert.{chunk_id}.1000.npy",
    y_path=lambda: f"{GS}/finefoods.labels.npy"
)
```

Here we are using only 400 chunks of all pre-computed chunks (that amounts to ~9 GB of our memeory).
In a bigger project, you would need to create your own generator that iterates over the chunks and do [lazy loading](https://en.wikipedia.org/wiki/Lazy_loading).
In theory, a small part of your data should be in memeory and your generator should load new chunks and deallocate the memory for the already-used chunks as it goes.

### Train a classifier

Let's train and test a two layer neural network classifer.

```
from tum.nlp import models

nn_model = models.DeepNeuralModel()
nn_model.train(data.train.X, data.train.y, data.test.X, data.test.y)
nn_model.test(data.test.X, data.test.y)
```

Note that we pass the test data to the training function because we run the test function on each epoch end. Under the hood, we are just using Keras's `fit_generator` funcitonality.

## Notes

- There are plenty of machine / deep learning platforms out there. We are not trying to introduce a new one.
- We are trying to come up with a good workflow that integrates the flexibility of using a Google Colab Notebook or a Jupyter Notebook and the convenience of having source-controlled code repository where multiple participants could contribute to the same code base.
"""
